@@ -13,13 +13,19 @@ class MoreSane : public DeconvolutionAlgorithm
 	public:
 		MoreSane(const std::string& moreSaneLocation, const std::string& moresaneArguments, 
 		         const ao::uvector<double> &moresaneSigmaLevels, const std::string &prefixName,
-						 ImageBufferAllocator& allocator) :
+						 ImageBufferAllocator& allocator, class FFTWManager& fftwManager) :
 			_moresaneLocation(moreSaneLocation), _moresaneArguments(moresaneArguments), 
 			_moresaneSigmaLevels(moresaneSigmaLevels), _prefixName(prefixName),
-			_allocator(&allocator)
+			_allocator(&allocator),
+			_fftwManager(fftwManager)
 		{ }
 		
-		virtual void ExecuteMajorIteration(ImageSet& dataImage, ImageSet& modelImage, const ao::uvector<const double*>& psfImages, size_t width, size_t height, bool& reachedMajorThreshold) final override;
+		virtual double ExecuteMajorIteration(ImageSet& dataImage, ImageSet& modelImage, const ao::uvector<const double*>& psfImages, size_t width, size_t height, bool& reachedMajorThreshold) final override;
+		
+		virtual std::unique_ptr<DeconvolutionAlgorithm> Clone() const final override
+		{
+			return std::unique_ptr<DeconvolutionAlgorithm>(new MoreSane(*this));
+		}
 		
 		void ExecuteMajorIteration(double* dataImage, double* modelImage, const double* psfImage, size_t width, size_t height);
 	private:
@@ -29,6 +35,7 @@ class MoreSane : public DeconvolutionAlgorithm
 		const std::string _prefixName;
 		
 		ImageBufferAllocator* _allocator;
+		class FFTWManager& _fftwManager;
 };
 
 #endif
