@@ -12,7 +12,23 @@
 #include <limits>
 #include <iostream>
 
-void FitsWriter::writeHeaders(fitsfile*& fptr, const std::string& filename, size_t nFreq, size_t nPol) const
+void FitsWriter::writeHeaders(fitsfile*& fptr, const std::string& filename) const
+{
+	if(_extraDimensions.empty())
+	{
+		std::vector<Dimension> dimensions(2);
+		dimensions[0].type = FrequencyDimension;
+		dimensions[0].size = 1;
+		dimensions[1].type = PolarizationDimension;
+		dimensions[1].size = 1;
+		writeHeaders(fptr, filename, dimensions);
+	}
+	else {
+		writeHeaders(fptr, filename, _extraDimensions);
+	}
+}
+
+void FitsWriter::writeHeaders(fitsfile *& fptr, const std::string& filename, const std::vector<Dimension>& extraDimensions) const
 {
 	int status = 0;
 	fits_create_file(&fptr, (std::string("!") + filename).c_str(), &status);
@@ -20,12 +36,12 @@ void FitsWriter::writeHeaders(fitsfile*& fptr, const std::string& filename, size
 	
 	// append image HDU
 	int bitPixInt = FLOAT_IMG;
-	long naxes[4];
+	std::vector<long> naxes(2 + extraDimensions.size());
 	naxes[0] = _width;
 	naxes[1] = _height;
-	naxes[2] = nFreq;
-	naxes[3] = nPol;
-	fits_create_img(fptr, bitPixInt, 4, naxes, &status);
+	for(size_t i=0; i!=extraDimensions.size(); ++i)
+		naxes[i+2] = extraDimensions[i].size;
+	fits_create_img(fptr, bitPixInt, naxes.size(), naxes.data(), &status);
 	checkStatus(status, filename);
 	double zero = 0, one = 1, equinox = 2000.0;
 	fits_write_key(fptr, TDOUBLE, "BSCALE", (void*) &one, "", &status); checkStatus(status, filename);
@@ -37,6 +53,9 @@ void FitsWriter::writeHeaders(fitsfile*& fptr, const std::string& filename, size
 		case JanskyPerBeam:
 			fits_write_key(fptr, TSTRING, "BUNIT", (void*) "JY/BEAM", "Units are in Jansky per beam", &status); checkStatus(status, filename);
 			break;
+		case JanskyPerPixel:
+			fits_write_key(fptr, TSTRING, "BUNIT", (void*) "JY/PIXEL", "Units are in Jansky per pixel", &status); checkStatus(status, filename);
+			break;
 		case Jansky:
 			fits_write_key(fptr, TSTRING, "BUNIT", (void*) "JY", "Units are in Jansky", &status); checkStatus(status, filename);
 			break;
@@ -118,36 +137,73 @@ void FitsWriter::writeHeaders(fitsfile*& fptr, const std::string& filename, size
 		fits_write_key(fptr, TSTRING, "CUNIT2", (void*) "deg", "", &status); checkStatus(status, filename);
 	}
 	
-
-	fits_write_key(fptr, TSTRING, "CTYPE3", (void*) "FREQ", "Central frequency", &status); checkStatus(status, filename);
-	fits_write_key(fptr, TDOUBLE, "CRPIX3", (void*) &one, "", &status); checkStatus(status, filename);
-	fits_write_key(fptr, TDOUBLE, "CRVAL3", (void*) &_frequency, "", &status); checkStatus(status, filename);
-	fits_write_key(fptr, TDOUBLE, "CDELT3", (void*) &_bandwidth, "", &status); checkStatus(status, filename);
-	fits_write_key(fptr, TSTRING, "CUNIT3", (void*) "Hz", "", &status); checkStatus(status, filename);
-	
-	double pol;
-	switch(_polarization)
+	char
+		ctypeDim[7] = "CTYPE?",
+		crpixDim[7] = "CRPIX?",
+		crvalDim[7] = "CRVAL?",
+		cdeltDim[7] = "CDELT?",
+		cunitDim[7] = "CUNIT?";
+	for(size_t i=0; i!=extraDimensions.size(); ++i)
 	{
-		case Polarization::StokesI: pol = 1.0; break;
-		case Polarization::StokesQ: pol = 2.0; break;
-		case Polarization::StokesU: pol = 3.0; break;
-		case Polarization::StokesV: pol = 4.0; break;
-		case Polarization::RR: pol = -1.0; break;
-		case Polarization::LL: pol = -2.0; break;
-		case Polarization::RL: pol = -3.0; break;
-		case Polarization::LR: pol = -4.0; break;
-		case Polarization::XX: pol = -5.0; break;
-		case Polarization::YY: pol = -6.0; break; //yup, this is really the right value
-		case Polarization::XY: pol = -7.0; break;
-		case Polarization::YX: pol = -8.0; break;
-		case Polarization::Instrumental:
-			throw std::runtime_error("Incorrect polarization given to fits writer");
+		ctypeDim[5] = (i+'3');
+		crpixDim[5] = (i+'3');
+		crvalDim[5] = (i+'3');
+		cdeltDim[5] = (i+'3');
+		cunitDim[5] = (i+'3');
+		switch(extraDimensions[i].type)
+		{
+		case FrequencyDimension:
+			fits_write_key(fptr, TSTRING, ctypeDim, (void*) "FREQ", "Central frequency", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TDOUBLE, crpixDim, (void*) &one, "", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TDOUBLE, crvalDim, (void*) &_frequency, "", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TDOUBLE, cdeltDim, (void*) &_bandwidth, "", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TSTRING, cunitDim, (void*) "Hz", "", &status); checkStatus(status, filename);
+			break;
+		case PolarizationDimension:
+			{
+				double pol;
+				switch(_polarization)
+				{
+					case Polarization::StokesI: pol = 1.0; break;
+					case Polarization::StokesQ: pol = 2.0; break;
+					case Polarization::StokesU: pol = 3.0; break;
+					case Polarization::StokesV: pol = 4.0; break;
+					case Polarization::RR: pol = -1.0; break;
+					case Polarization::LL: pol = -2.0; break;
+					case Polarization::RL: pol = -3.0; break;
+					case Polarization::LR: pol = -4.0; break;
+					case Polarization::XX: pol = -5.0; break;
+					case Polarization::YY: pol = -6.0; break; //yup, this is really the right value
+					case Polarization::XY: pol = -7.0; break;
+					case Polarization::YX: pol = -8.0; break;
+					case Polarization::Instrumental:
+						throw std::runtime_error("Incorrect polarization given to fits writer");
+				}
+				fits_write_key(fptr, TSTRING, ctypeDim, (void*) "STOKES", "", &status); checkStatus(status, filename);
+				fits_write_key(fptr, TDOUBLE, crpixDim, (void*) &one, "", &status); checkStatus(status, filename);
+				fits_write_key(fptr, TDOUBLE, crvalDim, (void*) &pol, "", &status); checkStatus(status, filename);
+				fits_write_key(fptr, TDOUBLE, cdeltDim, (void*) &one, "", &status); checkStatus(status, filename);
+				fits_write_key(fptr, TSTRING, cunitDim, (void*) "", "", &status); checkStatus(status, filename);
+			}
+			break;
+		case AntennaDimension:
+			fits_write_key(fptr, TSTRING, ctypeDim, (void*) "ANTENNA", "", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TDOUBLE, crpixDim, (void*) &one, "", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TDOUBLE, crvalDim, (void*) &zero, "", &status); checkStatus(status, filename);
+			break;
+		case TimeDimension:
+			fits_write_key(fptr, TSTRING, ctypeDim, (void*) "TIME", "", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TDOUBLE, crpixDim, (void*) &one, "", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TDOUBLE, crvalDim, (void*) &_timeDirectionStart, "", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TDOUBLE, cdeltDim, (void*) &_timeDirectionInc, "", &status); checkStatus(status, filename);
+			break;
+		case MatrixDimension:
+			fits_write_key(fptr, TSTRING, ctypeDim, (void*) "MATRIX", "", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TDOUBLE, crpixDim, (void*) &one, "", &status); checkStatus(status, filename);
+			fits_write_key(fptr, TDOUBLE, crvalDim, (void*) &zero, "", &status); checkStatus(status, filename);
+			break;
+		}
 	}
-	fits_write_key(fptr, TSTRING, "CTYPE4", (void*) "STOKES", "", &status); checkStatus(status, filename);
-	fits_write_key(fptr, TDOUBLE, "CRPIX4", (void*) &one, "", &status); checkStatus(status, filename);
-	fits_write_key(fptr, TDOUBLE, "CRVAL4", (void*) &pol, "", &status); checkStatus(status, filename);
-	fits_write_key(fptr, TDOUBLE, "CDELT4", (void*) &one, "", &status); checkStatus(status, filename);
-	fits_write_key(fptr, TSTRING, "CUNIT4", (void*) "", "", &status); checkStatus(status, filename);
 	
 	// RESTFRQ ?
 	fits_write_key(fptr, TSTRING, "SPECSYS", (void*) "TOPOCENT", "", &status); checkStatus(status, filename);
@@ -184,37 +240,31 @@ void FitsWriter::writeHeaders(fitsfile*& fptr, const std::string& filename, size
 	}
 }
 
-void FitsWriter::writeImage(fitsfile* fptr, const std::string& filename, const double* image) const
+void FitsWriter::writeImage(fitsfile* fptr, const std::string& filename, const double* image, long* currentPixel) const
 {
-	long firstpixel[4];
-	for(int i=0;i < 4;i++) firstpixel[i] = 1;
 	double nullValue = std::numeric_limits<double>::max();
 	int status = 0;
-	fits_write_pixnull(fptr, TDOUBLE, firstpixel, _width*_height, const_cast<double*>(image), &nullValue, &status);
+	fits_write_pixnull(fptr, TDOUBLE, currentPixel, _width*_height, const_cast<double*>(image), &nullValue, &status);
 	checkStatus(status, filename);
 }
 
-void FitsWriter::writeImage(fitsfile* fptr, const std::string& filename, const float* image) const
+void FitsWriter::writeImage(fitsfile* fptr, const std::string& filename, const float* image, long* currentPixel) const
 {
-	long firstpixel[4];
-	for(int i=0;i < 4;i++) firstpixel[i] = 1;
 	float nullValue = std::numeric_limits<float>::max();
 	int status = 0;
-	fits_write_pixnull(fptr, TFLOAT, firstpixel, _width*_height, const_cast<float*>(image), &nullValue, &status);
+	fits_write_pixnull(fptr, TFLOAT, currentPixel, _width*_height, const_cast<float*>(image), &nullValue, &status);
 	checkStatus(status, filename);
 }
 
 template<typename NumType>
-void FitsWriter::writeImage(fitsfile* fptr, const std::string& filename, const NumType* image) const
+void FitsWriter::writeImage(fitsfile* fptr, const std::string& filename, const NumType* image, long* currentPixel) const
 {
-	long firstpixel[4];
-	for(int i=0;i < 4;i++) firstpixel[i] = 1;
 	double nullValue = std::numeric_limits<double>::max();
 	int status = 0;
 	size_t totalSize = _width*_height;
 	std::vector<double> copy(totalSize);
 	for(size_t i=0;i!=totalSize;++i) copy[i] = image[i];
-	fits_write_pixnull(fptr, TDOUBLE, firstpixel, totalSize, &copy[0], &nullValue, &status);
+	fits_write_pixnull(fptr, TDOUBLE, currentPixel, totalSize, &copy[0], &nullValue, &status);
 	checkStatus(status, filename);
 }
 
@@ -231,9 +281,10 @@ void FitsWriter::Write(const std::string& filename, const NumType* image) const
 {
 	fitsfile *fptr;
 
-	writeHeaders(fptr, filename, 1, 1);
+	writeHeaders(fptr, filename);
 	
-	writeImage(fptr, filename, image);
+	long firstPixel[4] = { 1, 1, 1, 1};
+	writeImage(fptr, filename, image, firstPixel);
 	
 	int status = 0;
 	fits_close_file(fptr, &status);
@@ -244,12 +295,13 @@ template void FitsWriter::Write<long double>(const std::string& filename, const
 template void FitsWriter::Write<double>(const std::string& filename, const double* image) const;
 template void FitsWriter::Write<float>(const std::string& filename, const float* image) const;
 
-void FitsWriter::StartMulti(const std::string& filename, size_t nPol, size_t nFreq)
+void FitsWriter::StartMulti(const std::string& filename)
 {
 	if(_multiFPtr != 0)
 		throw std::runtime_error("StartMulti() called twice without calling FinishMulti()");
 	_multiFilename = filename;
-	writeHeaders(_multiFPtr, _multiFilename, 1, 1);
+	writeHeaders(_multiFPtr, _multiFilename, _extraDimensions);
+	_currentPixel.assign(_extraDimensions.size() + 2, 1);
 }
 
 void FitsWriter::FinishMulti()
