@@ -129,15 +129,6 @@ void Image::Trim(double* output, size_t outWidth, size_t outHeight, const double
 	}
 }
 
-void Image::TrimBox(bool* output, size_t x1, size_t y1, size_t boxWidth, size_t boxHeight, const bool* input, size_t inWidth, size_t /*inHeight*/)
-{
-	size_t endY = y1 + boxHeight;
-	for(size_t y=y1; y!=endY; ++y)
-	{
-		memcpy(&output[(y-y1)*boxWidth], &input[y*inWidth + x1], boxWidth*sizeof(bool));
-	}
-}
-
 /** Extend an image with zeros, complement of Trim.
 	* @param outWidth Should be &gt;= inWidth.
 	* @param outHeight Should be &gt;= inHeight.
