@@ -2,7 +2,7 @@
 
 #include "msprovider.h"
 
-void DirectMSRowProvider::ReadData(MSRowProvider::DataArray& data, MSRowProvider::FlagArray& flags, WeightArray& weights, double& u, double& v, double& w, uint32_t& dataDescId, uint32_t& antenna1, uint32_t& antenna2)
+void DirectMSRowProvider::ReadData(MSRowProvider::DataArray& data, MSRowProvider::FlagArray& flags, WeightArray& weights, double& u, double& v, double& w, uint32_t& dataDescId, uint32_t& antenna1, uint32_t& antenna2, double& time)
 {
 	u = _currentUVWArray(0);
 	v = _currentUVWArray(1);
@@ -12,6 +12,7 @@ void DirectMSRowProvider::ReadData(MSRowProvider::DataArray& data, MSRowProvider
 	_flagColumn.get(_currentRow, flags);
 	antenna1 = _antenna1Column(_currentRow);
 	antenna2 = _antenna2Column(_currentRow);
+	time = _timeColumn(_currentRow);
 	
 	getCurrentWeights(weights);
 }
