@@ -2,9 +2,9 @@
 
 #include "multiscaletransforms.h"
 
-#include "../deconvolution/clarkloop.h"
 #include "../deconvolution/componentlist.h"
 #include "../deconvolution/simpleclean.h"
+#include "../deconvolution/subminorloop.h"
 
 #include "../fftwmanager.h"
 
@@ -12,14 +12,11 @@
 
 #include "../units/fluxdensity.h"
 
-
 MultiScaleAlgorithm::MultiScaleAlgorithm(ImageBufferAllocator& allocator, FFTWManager& fftwManager, double beamSize, double pixelScaleX, double pixelScaleY) :
 	_allocator(allocator),
 	_fftwManager(fftwManager),
 	_width(0),
 	_height(0),
-	_convolutionWidth(0),
-	_convolutionHeight(0),
 	_convolutionPadding(1.1),
 	_beamSizeInPixels(beamSize / std::max(pixelScaleX, pixelScaleY)),
 	_multiscaleScaleBias(0.6),
@@ -71,12 +68,6 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 		_allowNegativeComponents = true;
 	_width = width;
 	_height = height;
-	_convolutionWidth = ceil(_convolutionPadding * _width);
-	_convolutionHeight = ceil(_convolutionPadding * _height);
-	if(_convolutionWidth%2 != 0)
-		++_convolutionWidth;
-	if(_convolutionHeight%2 != 0)
-		++_convolutionHeight;
 	// The threads always need to be stopped at the end of this function, so we use a scoped
 	// unique ptr.
 	std::unique_ptr<ThreadedDeconvolutionTools> tools(new ThreadedDeconvolutionTools(_threadCount));
@@ -87,7 +78,7 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 	{
 		// Note that in a second round the nr of scales can be different (due to different width/height,
 		// e.g. caused by a different subdivision in parallel cleaning).
-		for(const ao::uvector<bool>& mask :_scaleMasks)
+		for(const ao::uvector<bool>& mask : _scaleMasks)
 		{
 			if(mask.size() != _width*_height)
 				throw std::runtime_error("Invalid automask size in multiscale algorithm");
@@ -97,17 +88,29 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 			_scaleMasks.emplace_back(_width*_height, false);
 		}
 	}
-	if(_trackComponents && _componentList == nullptr)
+	if(_trackComponents)
 	{
-		_componentList.reset(new ComponentList(_width, _height, _scaleInfos.size(), dirtySet.size(), _allocator));
+		if(_componentList == nullptr)
+			_componentList.reset(new ComponentList(_width, _height, _scaleInfos.size(), dirtySet.size(), _allocator));
+		else if(_componentList->Width() != _width || _componentList->Height() != _height)
+		{
+			throw std::runtime_error("Error in component list dimensions!");
+		}
 	}
+	if(!_rmsFactorImage.empty() && (_rmsFactorImage.Width() != _width || _rmsFactorImage.Height() != _height))
+			throw std::runtime_error("Error in RMS factor image dimensions!");
 	
 	bool hasHitThresholdInSubLoop = false;
 	size_t thresholdCountdown = std::max(size_t(8), _scaleInfos.size()*3/2);
 	
 	ImageBufferAllocator::Ptr scratch, scratchB, integratedScratch;
-	_allocator.Allocate(_convolutionWidth*_convolutionHeight, scratch);
-	_allocator.Allocate(_convolutionWidth*_convolutionHeight, scratchB);
+	// scratch and scratchB are used by the subminorloop, which convolves the images
+	// and requires therefore more space. This space depends on the scale, so here
+	// the required size for the largest scale is calculated.
+	size_t scratchWidth, scratchHeight;
+	getConvolutionDimensions(_scaleInfos.size()-1, scratchWidth, scratchHeight);
+	_allocator.Allocate(scratchWidth*scratchHeight, scratch);
+	_allocator.Allocate(scratchWidth*scratchHeight, scratchB);
 	_allocator.Allocate(_width*_height, integratedScratch);
 	std::unique_ptr<std::unique_ptr<ImageBufferAllocator::Ptr[]>[]> convolvedPSFs(
 		new std::unique_ptr<ImageBufferAllocator::Ptr[]>[dirtySet.PSFCount()]);
@@ -158,8 +161,7 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 		_allocator.Allocate(_width*_height, doubleConvolvedPSFs[i]);
 	}
 	
-	ImageSet individualConvolvedImages(&dirtySet.Table(), dirtySet.Allocator(), dirtySet.ChannelsInDeconvolution(), dirtySet.SquareJoinedChannels(),
-	dirtySet.LinkedPolarizations(), _width, _height);
+	ImageSet individualConvolvedImages(&dirtySet.Table(), dirtySet.Allocator(), dirtySet.Settings(), _width, _height);
 	
 	//
 	// The minor iteration loop
@@ -213,7 +215,9 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 		{
 			FFTWManager::ThreadingScope fftwMT(_fftwManager);
 			size_t subMinorStartIteration = _iterationNumber;
-			ClarkLoop clarkLoop(_width, _height, _convolutionWidth, _convolutionHeight);
+			size_t convolutionWidth, convolutionHeight;
+			getConvolutionDimensions(scaleWithPeak, convolutionWidth, convolutionHeight);
+			SubMinorLoop clarkLoop(_width, _height, convolutionWidth, convolutionHeight);
 			clarkLoop.SetIterationInfo(_iterationNumber, MaxNIter());
 			clarkLoop.SetThreshold(firstSubIterationThreshold / _scaleInfos[scaleWithPeak].biasFactor, subIterationGainThreshold / _scaleInfos[scaleWithPeak].biasFactor);
 			clarkLoop.SetGain(_scaleInfos[scaleWithPeak].gain);
@@ -337,9 +341,9 @@ double MultiScaleAlgorithm::ExecuteMajorIteration(ImageSet& dirtySet, ImageSet&
 
 void MultiScaleAlgorithm::initializeScaleInfo()
 {
-	if(_scaleInfos.empty())
+	if(_manualScaleList.empty())
 	{
-		if(_manualScaleList.empty())
+		if(_scaleInfos.empty())
 		{
 			size_t scaleIndex = 0;
 			double scale = _beamSizeInPixels * 2.0;
@@ -357,16 +361,24 @@ void MultiScaleAlgorithm::initializeScaleInfo()
 			} while(scale < std::min(_width, _height)*0.5);
 		}
 		else {
-			std::sort(_manualScaleList.begin(), _manualScaleList.end());
-			for(size_t scaleIndex = 0; scaleIndex != _manualScaleList.size(); ++scaleIndex)
+			while(!_scaleInfos.empty() && _scaleInfos.back().scale >= std::min(_width, _height) * 0.5)
 			{
-				_scaleInfos.push_back(ScaleInfo());
-				ScaleInfo& newEntry = _scaleInfos.back();
-				newEntry.scale = _manualScaleList[scaleIndex];
-				newEntry.kernelPeak = MultiScaleTransforms::KernelPeakValue(newEntry.scale, std::min(_width, _height), _scaleShape);
+				Logger::Info << "Scale size " << _scaleInfos.back().scale << " does not fit in cleaning region: removing scale.\n";
+				_scaleInfos.erase(_scaleInfos.begin() + _scaleInfos.size() - 1);
 			}
 		}
 	}
+	if(!_manualScaleList.empty() && _scaleInfos.empty())
+	{
+		std::sort(_manualScaleList.begin(), _manualScaleList.end());
+		for(size_t scaleIndex = 0; scaleIndex != _manualScaleList.size(); ++scaleIndex)
+		{
+			_scaleInfos.push_back(ScaleInfo());
+			ScaleInfo& newEntry = _scaleInfos.back();
+			newEntry.scale = _manualScaleList[scaleIndex];
+			newEntry.kernelPeak = MultiScaleTransforms::KernelPeakValue(newEntry.scale, std::min(_width, _height), _scaleShape);
+		}
+	}
 }
 
 void MultiScaleAlgorithm::convolvePSFs(std::unique_ptr<ImageBufferAllocator::Ptr[]>& convolvedPSFs, const double* psf, double* tmp, bool isIntegrated)
@@ -589,3 +601,21 @@ void MultiScaleAlgorithm::findPeakDirect(const double* image, double* scratch, s
 	else
 		scaleInfo.maxNormalizedImageValue = get_optional_value_or(maxValue, 0.0) / _rmsFactorImage[scaleInfo.maxImageValueX + scaleInfo.maxImageValueY * _width];
 }
+
+void MultiScaleAlgorithm::getConvolutionDimensions(size_t scaleIndex, size_t& width, size_t& height) const
+{
+	double scale = _scaleInfos[scaleIndex].scale;
+	// The factor of 1.5 comes from some superficial experience with diverging runs.
+	// It's supposed to be a balance between diverging runs caused by
+	// insufficient padding on one hand, and taking up too much memory on the other.
+	// I've seen divergence when padding=1.1, _width=1500, max scale=726 and conv width=1650.
+	// Diverged occurred on scale 363.
+	// Was solved with conv width=2250. 2250 = 1.1*(363*factor + 1500)  --> factor = 1.5
+	// And solved with conv width=2000. 2000 = 1.1*(363*factor + 1500)  --> factor = 0.8
+	width = ceil(_convolutionPadding * (scale*1.5 + _width));
+	height = ceil(_convolutionPadding * (scale*1.5 + _height));
+	if(width%2 != 0)
+		++width;
+	if(height%2 != 0)
+		++height;
+}
