@@ -1,5 +1,6 @@
 #include "wsclean/commandline.h"
 #include "wsclean/logger.h"
+#include "wsclean/wsclean.h"
 
 #include <exception>
 #include <iostream>
@@ -7,8 +8,10 @@
 int main(int argc, char *argv[])
 {
 	try {
-		int returnValue = CommandLine::Run(argc, argv);
-		return returnValue;
+		WSClean wsclean;
+		if(CommandLine::Parse(wsclean, argc, argv))
+			CommandLine::Run(wsclean);
+		return 0;
 	} catch(std::exception& e)
 	{
 		Logger::Error
